/*
 * Copyright 2006-2007, Thomas Maier-Komor
 * LICENSE: GPLv3, see file COPYING for details
 */

#include <assert.h>
#include <stdlib.h>
#include <strings.h>
#include "userstat.h"

static int sort_nproc(userstat_t *u1, userstat_t *u2);
static int (*Crit)(userstat_t *, userstat_t *) = sort_nproc;

static int 
sort_pcpu(userstat_t *u1, userstat_t *u2)
{
	return u1->pctcpu < u2->pctcpu;
}


static int 
sort_user(userstat_t *u1, userstat_t *u2)
{
	return strcmp(u1->username,u2->username) > 0;
}


static int 
sort_nproc(userstat_t *u1, userstat_t *u2)
{
	if (u1->nproc == u2->nproc)
		return sort_user(u1,u2);
	return u1->nproc < u2->nproc;
}


static int 
sort_rss(userstat_t *u1, userstat_t *u2)
{
	return u1->rss < u2->rss;
}


static int 
sort_size(userstat_t *u1, userstat_t *u2)
{
	return u1->size < u2->size;
}


static int 
sort_hs(userstat_t *u1, userstat_t *u2)
{
	if (u1->h_s)
		return u1->h_s < u2->h_s;
	return u1->rss < u2->rss;
}


static int 
sort_time(userstat_t *u1, userstat_t *u2)
{
	return u1->time.tv_sec < u2->time.tv_sec;
}


void 
set_sort_by_pcpu(void)
{
	Crit = sort_pcpu;
}


void 
set_sort_by_nproc(void)
{
	Crit = sort_nproc;
}


void 
set_sort_by_rss(void)
{
	Crit = sort_rss;
}


void
set_sort_by_size(void)
{
	Crit = sort_size;
}


void
set_sort_by_time(void)
{
	Crit = sort_time;
}


void
set_sort_by_hs(void)
{
	Crit = sort_hs;
}


void
set_sort_by_user(void)
{
	Crit = sort_user;
}


userstat_t *
sort_users(userstat_t *u)
{
	userstat_t *h = 0, *t = 0;
	while (u) {
		userstat_t *n = u->next, *i = h;
		if (u->nproc == 0) {
			free(u->username);
			free(u);
			u = n;
			continue;
		}
		while (i && Crit(u,i)
			&& i->next && Crit(u,i->next)) {
			assert(i != i->next);
			i = i->next;
		}
		if (i == 0) {
			if (t) {
				assert(h && !Crit(t, i));
				t->next = u;
				t = u;
			} else {
				assert(h == 0);
				h = t = u;
			}
			u->next = 0;
		} else {
			if (Crit(u,i)) {
				u->next = i->next;
				i->next = u;
			} else {
				u->next = i;
				h = u;
			}
			assert(i != i->next);
		}
		u = n;
		assert(u == 0 || u != u->next);
	}
	return h;
}


// vim:tw=0
