/*
 * Copyright 2006-2008, Thomas Maier-Komor
 * LICENSE: GPLv3, see file COPYING for details
 */

#include "sysstat.h"
#include "support.h"

#include <stdlib.h>
#include <strings.h>
#include <unistd.h>


static char *text = 0, *e = 0;

static char *adjust_buffer(char *t)
{
	if (t + 81 >= e) {
		size_t newsize, off = t - text;
		char *nb;
		newsize = e-text + 100 * 81;
		nb = realloc(text,newsize);
		if (nb == 0) {
			nb = malloc(newsize);
			memcpy(nb,text,e-text);
			free(text);
		}
		text = nb;
		e = text + newsize;
		t = text + off;
	}
	return t;

}

static void
display_sysstat(sysstat_t *s)
{
	userstat_t *u;
	fsstat_t *fs;
	char Time[32], *t = text;
	unsigned numuser = 0;
	diskrates_t *dr = s->diskrates;
	int longdevs = 0;

	if (s == 0)
		return;
	if (text == 0) {
		text = malloc(81*25 +1);
		assert(text);
		e = text + 81*25 + 1;
		t = text;
	}
	*t++ = '\n';
	*t++ = '\n';
	t += sprintf(t,s->Hostinfo);
	if (s->Ocpus == s->Ncpus) {
		if (s->Ncpus > 1) {
			t += sprintf(t,", %u CPUs", s->Ocpus);
		} else {
			assert(s->Ocpus == 1);
			t += sprintf(t,", 1 CPU");
		}
	} else {
		assert(s->Ncpus > 1);
		t += sprintf(t,", %u/%u CPUs",s->Ocpus,s->Ncpus);
	}
	if (s->BootTime) {
		int d,h,m,b;
		b = s->TS - s->BootTime;
		d = b / (60*60*24);
		b -= d * (60*60*24);
		h = b / (60*60);
		b -= h * (60*60);
		m = b / (60);
		b -= m * (60);
		if (d == 1)
			t += sprintf(t,", up 1 day");
		else if (d)
			t += sprintf(t,", up %d days",d);
		else
			t += sprintf(t,", up %d:%02d:%02d",h,m,b);
	} else if (s->Devices != 1) {
		t += sprintf(t,", %u disks  ", s->Devices);
	} else {
		t += sprintf(t,", 1 disk  ");
	}
	strftime(Time,sizeof(Time),"%c",localtime(&s->TS));
	t += sprintf(t,"  %.20s\n",Time);
	memcpy(t,"--------------------------------------------------------------------------------\n",81);
	t += 81;

	t += sprintf(t," User   CPU: %s%% | ", per2str(s->UserP));
	t += sprintf(t,"LoadAv/1m:  %4.2f  | ", s->loadavg[0]);
	t += sprintf(t,"Physical:    %s | ", kb2str(s->Physmem));
	if (s->NetInB >= 0) 
		t += sprintf(t,"Net in/s:   %s \n", kb2str(s->NetInB));
	else if (s->IpInR >= 0)
		t += sprintf(t,"IP rcv/s:   %s \n", dbl2str(s->IpInR));
	else
		t += sprintf(t,"ipacket/s:  %s \n", dbl2str(s->NetInR));

	t += sprintf(t," Kernel CPU: %s%% | ",per2str(s->KernelP));
	t += sprintf(t,"LoadAv/5m:  %4.2f  | ", s->loadavg[1]);
	t += sprintf(t,"Free:        %s | ", kb2str(s->Pagesfree));
	if (s->NetInB >= 0) 
		t += sprintf(t,"Net out/s:  %s \n", kb2str(s->NetOutB));
	else if (s->IpOutR >= 0)
		t += sprintf(t,"IP send/s:  %s \n", dbl2str(s->IpOutR));
	else
		t += sprintf(t,"opacket/s:  %s \n", dbl2str(s->NetOutR));

	t += sprintf(t," Idle   CPU: %s%% | ",per2str(s->IdleP));
	t += sprintf(t,"LoadAv/15m: %4.2f  | ", s->loadavg[2]);
	t += sprintf(t,"Locked:      %s | ", kb2str(s->Pageslocked));
	t += sprintf(t,"net errs: %6lu \n", s->IpInDiscards+s->IpOutDiscards+s->IpInErrs);
	memcpy(t,"--------------------------------------------------------------------------------\n",81);
	t += 81;
	t += sprintf(t," intr/s:     %s | ", dbl2str(s->IntrR));
	t += sprintf(t,"syscall/s:  %s | ", dbl2str(s->SyscallR));
	if (s->ZfsArcSize > 0) 
		t += sprintf(t,"ZfsArcSize:  %s | ", kb2str(s->ZfsArcSize));
	else 
		t += sprintf(t,"SwapTotal:   %s | ", kb2str(s->SwapTotal));
	t += sprintf(t,"nfsd ops/s: %s \n", dbl2str(s->NfsdR));

	t += sprintf(t," intrblk/s:  %s | ", dbl2str(s->IntrBlkR));
	t += sprintf(t,"sysfork/s:  %s | ", dbl2str(s->SysforkR));
	t += sprintf(t,"SwapFree:    %s | ", kb2str(s->SwapFree));
	t += sprintf(t,"NfsIn/s:    %s \n", kb2str(s->NfsIR));

	t += sprintf(t," xcalls/s:   %s | ", dbl2str(s->XcallsR));
	t += sprintf(t,"sysexec/s:  %s | ", dbl2str(s->SysexecR));
	t += sprintf(t,"scan/s:      %s | ",dbl2str(s->ScanR));
	t += sprintf(t,"NfsOut/s:   %s \n", kb2str(s->NfsOR));

	t += sprintf(t," pswitch/s:  %s | ", dbl2str(s->PswitchR));
	t += sprintf(t,"sysread/s:  %s | ", dbl2str(s->SysreadR));
	if ((s->PgswapinR != 0) && (s->AnonpginR == 0))
		t += sprintf(t,"pgswapin/s:  %s | ", dbl2str(s->PgswapinR));
	else
		t += sprintf(t,"anonpgin/s:  %s | ", dbl2str(s->AnonpginR));
	t += sprintf(t,"DiskIn/s:   %s \n", kb2str(s->DiskIR));

	t += sprintf(t," migrate/s:  %s | ", dbl2str(s->MigrateR));
	t += sprintf(t,"syswrite/s: %s | ", dbl2str(s->SyswriteR));
	if ((s->PgswapoutR != 0) && (s->AnonpgoutR == 0))
		t += sprintf(t,"pgswapout/s: %s | ", dbl2str(s->PgswapoutR));
	else
		t += sprintf(t,"anonpgout/s: %s | ", dbl2str(s->AnonpgoutR));
	t += sprintf(t,"DiskOut/s:  %s \n", kb2str(s->DiskOR));

	t += sprintf(t," invswtch/s: %s | ", dbl2str(s->InvSwtchR));
	t += sprintf(t,"                  | ");
	t += sprintf(t,"anonfree/s   %s | ",dbl2str(s->AnonfreeR));
	//t += sprintf(t,"FsFree:      %s | ", kb2str(s->FsFree));
	t += sprintf(t,"disk errs: %5lu  \n", s->HardErrors+s->SoftErrors);

	memcpy(t,"--------------------------------------------------------------------------------\n",81);
	t += 81;
	if (s->cpurates) {
		cpurates_t *r = s->cpurates;
		memcpy(t,"CPU User/Syst/Idle | xcal intr ithr iblk  csw icsw migr smtx  srw sysc minf majf\n",81);
		t += 81;
		do {
			t += sprintf(t,"%3d",r->id);
			*t++ = ' ';
			t = Per2str(r->user,t);
			*t++ = ' ';
			t = Per2str(r->kernel,t);
			*t++ = ' ';
			t = Per2str(r->idle,t);
			*t++ = ' ';
			*t++ = '|';
			*t++ = ' ';
			t = Dbl2str3(r->xcalls,t);
			*t++ = ' ';
			t = Dbl2str3(r->intr,t);
			*t++ = ' ';
			t = Dbl2str3(r->intrthread,t);
			*t++ = ' ';
			t = Dbl2str3(r->intrblk,t);
			*t++ = ' ';
			t = Dbl2str3(r->pswitch,t);
			*t++ = ' ';
			t = Dbl2str3(r->inv_swtch,t);
			*t++ = ' ';
			t = Dbl2str3(r->migrate,t);
			*t++ = ' ';
			t = Dbl2str3(r->mxae,t);
			*t++ = ' ';
			t = Dbl2str3(r->rwrf+r->rwwf,t);
			*t++ = ' ';
			t = Dbl2str3(r->syscall,t);
			*t++ = ' ';
			t = Dbl2str3(r->as_fault+r->hat_fault,t);
			*t++ = ' ';
			t = Dbl2str3(r->majfault,t);
			*t++ = '\n';
			r = r->next;
			t = adjust_buffer(t);
		} while(r);
		memcpy(t,"--------------------------------------------------------------------------------\n",81);
		t += 81;
	}

	u = s->us;
	while (u) {
		++numuser;
		u = u->next;
	}
	t = adjust_buffer(t);
	u = s->us;
	t += sprintf(t,"%d",numuser);
	if (u->h_s) {
		if (numuser < 10)
			t += strlcpy(t," User   NPROC ( O/ S/ R/ Z/ T) NLWP   SIZE   RSS    H+S   %MEM  %CPU   CPU-TIME\n",e - t);
		else if (numuser < 100)
			t += strlcpy(t," User  NPROC ( O/ S/ R/ Z/ T) NLWP   SIZE   RSS    H+S   %MEM  %CPU   CPU-TIME\n",e - t);
		else 
			t += strlcpy(t," User NPROC ( O/ S/ R/ Z/ T) NLWP   SIZE   RSS    H+S   %MEM  %CPU   CPU-TIME\n",e - t);
	} else {
		if (numuser < 10)
			t += strlcpy(t," User         NPROC ( O/ S/ R/ Z/ T) NLWP   SIZE   RSS   %MEM  %CPU    CPU-TIME\n",e - t);
		else if (numuser < 100)
			t += strlcpy(t," User        NPROC ( O/ S/ R/ Z/ T) NLWP   SIZE   RSS   %MEM  %CPU    CPU-TIME\n",e - t);
		else 
			t += strlcpy(t," User       NPROC ( O/ S/ R/ Z/ T) NLWP   SIZE   RSS   %MEM  %CPU    CPU-TIME\n",e - t);
	}

	while (u) {
		int hr,mn,sc;
		double pctmem,pctcpu;
		t = adjust_buffer(t);
		sc = u->time.tv_sec;
		sc += rint((double)u->time.tv_nsec/1000000000);
		mn = sc / 60;
		sc -= mn * 60;
		hr = mn / 60;
		mn -= hr * 60;
		if (u->h_s)
			t += sprintf(t,"%-9s ", u->username);
		else
			t += sprintf(t,"%-15s ", u->username);
		t += sprintf(t,"%4lu (%2lu/%2lu/%2lu/%2lu/%2lu) %4lu  %s  ",
				u->nproc,
				u->oncpu,
				u->sleeping,
				u->runnable,
				u->zombie,
				u->stopped,
				u->nlwp,
				kb2str(u->size));
		t += sprintf(t,"%s  ",kb2str(u->rss));
		if (u->h_s)
			t += sprintf(t,"%s  ",kb2str(u->h_s));
		pctmem = (double)u->pctmem/(double)0x8000*100.0;
		if (pctmem < 9.995)
			t += sprintf(t,"%3.1f%%  ",pctmem);
		else
			t += sprintf(t,"%3.f%%  ",pctmem);
		pctcpu = (double)u->pctcpu/(double)0x8000*100.0;
		if (pctcpu < 9.995)
			t += sprintf(t,"%3.1f%% ",pctcpu);
		else
			t += sprintf(t,"%3.f%% ",pctcpu);
		t += sprintf(t,"%4d:%02d:%02d\n",hr,mn,sc);
		u = u->next;
	}
	t = adjust_buffer(t);

	memcpy(t,"--------------------------------------------------------------------------------\n",81);
	t += 81;
	do {
		if (strlen(dr->device) > 11) {
			longdevs = 1;
			break;
		}
		dr = dr->next;
	} while (dr);
	if (longdevs)
		memcpy(t,"Device                                               op/s  rd/s  wr/s   %w   %b \n",81);
	else
		memcpy(t,"Device      op/s  rd/s  wr/s   %w   %b | Device      op/s  rd/s  wr/s   %w   %b \n",81);
	t += 81;
	dr = s->diskrates;
	while (dr) {
		if (longdevs)
			t += sprintf(t,"%-52.52s ",dr->device);
		else
			t += sprintf(t,"%-11.11s ",dr->device);
		t = Dbl2str3(dr->reads+dr->writes,t);
		*t++ = ' ';
		t = Kb2str(dr->nread,t);
		*t++ = ' ';
		t = Kb2str(dr->nwritten,t);
		*t++ = ' ';
		t = Per2str(dr->wbusy,t);
		*t++ = ' ';
		t = Per2str(dr->rbusy,t);
		*t++ = ' ';
		dr = dr->next;
		if (longdevs) {
			t = adjust_buffer(t);
			continue;
		}
		*t++ = '|';
		*t++ = ' ';
		if (dr) {
			t += sprintf(t,"%-11.11s ",dr->device);
			t = Dbl2str3(dr->reads+dr->writes,t);
			*t++ = ' ';
			t = Kb2str(dr->nread,t);
			*t++ = ' ';
			t = Kb2str(dr->nwritten,t);
			*t++ = ' ';
			t = Per2str(dr->wbusy,t);
			*t++ = ' ';
			t = Per2str(dr->rbusy,t);
			*t++ = '\n';
			dr = dr->next;
		} else 
			t += sprintf(t,"\n");
		t = adjust_buffer(t);
	}
	memcpy(t,"--------------------------------------------------------------------------------\n",81);
	t += 81;
	t = adjust_buffer(t);
	memcpy(t,"device                typ   mountpoint            ops/s  in/s out/s  avail/ size\n",81);
	t += 81;
	
	for (fs = s->fs; fs; fs = fs->next) {
		const char *f;
		t = adjust_buffer(t);
		if (strlen(fs->dev) > 21)
			f = "%.18s... %-5s ";
		else
			f = "%-21s %-5s ";
		t += sprintf(t,f,fs->dev,fs->typ);
		if (strlen(fs->mnt) > 21)
			f = "%.18s... %s";
		else
			f = "%-21s %s";
		t += sprintf(t,f,fs->mnt,dbl2str(fs->ops_rate));
		t += sprintf(t," %s/",kb2str(fs->read_rate));
		t += sprintf(t,"%s ",kb2str(fs->write_rate));
		t += sprintf(t," %s/",kb2str(fs->free));
		t += sprintf(t,"%s\n",kb2str(fs->blocks));
	}

	t = adjust_buffer(t);
	memcpy(t,"================================================================================",80);
	t += 80;
	(void) write(STDOUT_FILENO,text,t - text);
}


unsigned display_stdout(sysstat_t *s)
{
	unsigned numdis = 0;
	sysstat_t *f = s;
	if (s == 0)
		return 0;
	do {
		if (s->displayed == 0) {
			display_sysstat(s);
			s->displayed = 1;
			++numdis;
		}
		s = s->next;
	} while (s != f);
	return numdis;
}

// vim:tw=0
