/*
 * Copyright 2006-2009, Thomas Maier-Komor
 * LICENSE: GPLv3, see file COPYING for details
 */

#ifndef SUPPORT_H
#define SUPPORT_H

#include <assert.h>
#include <math.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>


static inline char *
Kb2str(double v, char *s)
{
	const char *dim = "kMGT", *f;

	while (v > 10000) {
		v /= 1024.0;
		++dim;
		if (*dim == 0) {
			v *= 1024.0*1024.0*1024.0*1024.0;
			break;
		}
	}
	if (v < 0)
		f = " ??? ";
	else if (rint(v) < 100)
		f = "%4.1f%c";
	else if (rint(v) < 10000)
		f = "%4.0f%c";
	else
		f = "%5.g";
	(void) sprintf(s,f,v,*dim);
	return s + 5;
}


static inline char *
kb2str(double v)
{
	static char str[8];
	Kb2str(v,str);
	return str;
}


static inline char *
Per2str(double d, char *s)
{
	const char *f;
	assert(d <= 1E2);
	if (d < 0) {
		f = "n/a ";
	} else if (d < 2E-2) {
		f = "%4.0f";
	} else if (d < 9.995) {
		f = "%4.2f";
	} else if (d < 99.95) {
		f = "%4.1f";
	} else {
		f = "%4.0f";
		d = rint(d);
	}
	(void) sprintf(s,f,d);
	return s + 4;
}

static inline char *
per2str(double d)
{
	static char str[8];
	const char *f;
	assert(d <= 1E2);
	if (d < 0) {
		f = "n/a ";
	} else if (d < 2E-2) {
		f = "%4.0f";
	} else if (d < 9.995) {
		f = "%4.2f";
	} else if (d < 99.95) {
		f = "%4.1f";
	} else {
		f = "%4.0f";
		d = rint(d);
	}
	(void) sprintf(str,f,d);
	return str;
}


static inline char *
Dbl2str3(double d, char *s)
{
	const char *dim = " kMGT", *f;
	double r = d;

	while (r >= 1E4) {
		r /= 1E3;
		++dim;
		if (*dim == 0) {
			r *= 1E15;
			d = -1;
			break;
		}
	}
	if (*dim == ' ') {
		if (r < 1E-2)
			f = "%4.0f";
		else if (r < 9.995)
			f = "%4.2f";
		else if (r < 1E2)
			f = "%4.1f";
		else if (r < 1E4)
			f = "%4.0f";
		else
			abort();
	} else if (r < 1E-2)
		f = "%3.0f%c";
	else if (r < 9.995)
		f = "%3.1f%c";
	else if (r < 1E4)
		f = "%3.0f%c";
	else
		f = "%4.g";
	(void) sprintf(s,f,r,*dim);
	return s + 4;
}


static inline char *
Dbl2str(double d, char *s)
{
	const char *dim = " kMGT", *f;
	double r = d;

	while (r >= 1E4) {
		r /= 1E3;
		++dim;
		if (*dim == 0) {
			r *= 1E15;
			d = -1;
			break;
		}
	}
	if (d < 0)
		f = "unav.";
	else if (r < 2E-2)
		f = "%4.0f%c";
	else if (r < 9.995)
		f = "%4.2f%c";
	else if (r < 1E2)
		f = "%4.1f%c";
	else if (r < 1E4)
		f = "%4.0f%c";
	else
		f = "%5.g";
	(void) sprintf(s,f,r,*dim);
	return s + 5;
}


static inline char *
dbl2str(double d)
{
	static char str[8];
	Dbl2str(d,str);
	return str;
}

#if SOLARIS < 8 && !defined(__CYGWIN__)
#ifndef STRLCPY_DEFINITION
#define STRLCPY_DEFINITION
static inline size_t strlcpy(char *dst, const char *src, size_t dstsize)
{
	size_t l = strlen(src);
	if (l >= dstsize)
		strncpy(dst,src,dstsize-1);
	else
		strcpy(dst,src);
	return l;
}
#endif
#endif

#endif
