/*
 * Copyright 2006-2009, Thomas Maier-Komor
 * LICENSE: GPLv3, see file COPYING for details
 */

#ifndef SYSSTAT_H
#define SYSSTAT_H

#include "fsstats.h"
#include "userstat.h"

#define VERSION "20091024"

#ifdef __STDC__
#if __STDC_VERSION__ < 199901L
#define restrict
#endif
#else
#define restrict
#endif


typedef struct cpurates_s {
	struct cpurates_s *next;
	double
		user, kernel, idle, wait,
		intr, intrthread, intrblk, xcalls,
		syscall, sysfork, sysvfork, sysexec,
		sysread, syswrite,
		pgswapin, pgswapout, anonpgin, anonpgout,
		as_fault, hat_fault, majfault,
		scan, anonfree,
		rwrf, rwwf, mxae,
		inv_swtch, pswitch, migrate;
	int	id;
} cpurates_t;

typedef struct netrates_s {
	struct netrates_s *next;
	char	device[32];
	double	inbs, outbs, ipckts, opckts;
} netrates_t;


typedef struct diskrates_s {
	struct diskrates_s *next;
	int	major;
	double
		rcnt, wcnt, reads, writes, nread, nwritten,
		rlentime, wlentime, rbusy, wbusy;
	char	device[32],disk[32];
} diskrates_t;

typedef struct sysstat_s {
	struct sysstat_s *next, *prev;
	char Hostinfo[256];
	clock_t BootTime;
	unsigned Ocpus, Ncpus, Devices;
	clock_t TS;
	/* system load */
	double loadavg[3];
	/* cpu/sys percentages */
	double UserP, KernelP, IdleP;
	/* cpu/sys rates */
	double IntrR, IntrThreadR, IntrBlkR, XcallsR, LockfR,
	       SyscallR, SysexecR, SysforkR, SysreadR, SyswriteR;
	/* cpu/vm rates */
	double ExecpginR, ExecpgoutR, ExecfreeR,
	       AnonpginR, AnonpgoutR, AnonfreeR,
	       InvSwtchR, PswitchR, MigrateR,
	       PgswapinR, PgswapoutR, ScanR,
	       AsFaultR, HatFaultR, MajFaultR;
	/* ip */
	double IpInR, IpOutR, NetInR, NetOutR, NetInB, NetOutB;
	unsigned long IpInDiscards, IpOutDiscards, IpInErrs;
	/* storage */
	unsigned long long FsFree, Storage, Zsize, Zfree, ZfsArcSize;
	unsigned long Zpools;
	/* unix/system_pages */
	unsigned long Pgsz;
	unsigned long Physmem, Pagesfree, Pageslocked, Pagestotal;
	/* swap (non-kstat info) */
	unsigned long SwapTotal, SwapFree;
	/* device_errors */
	unsigned long HardErrors, SoftErrors;
	/* device I/O rates */
	double DiskIR, DiskOR;
	/* nfs I/O rates */
	double NfsIR, NfsOR;
	/* nfsd call rate */
	double NfsdR;

	/* detailed CPU statistics */
	cpurates_t *cpurates;
	/* detailed disk statistics */
	diskrates_t *diskrates;
	/* user statistics */
	userstat_t *us;
	/* filesystem statistics */
	fsstat_t *fs;
	/* network statistics */
	netrates_t *netrates;
	unsigned numfs,numdisks;

	/* used locally, to determine if stat has already been displayed */
	int displayed;
	/* support lists of sysstats */
} sysstat_t;

extern sysstat_t Sysstat;

extern void rundaemon(unsigned interval, unsigned fsrate, unsigned userrate, unsigned count);

extern unsigned Interval;

#ifdef __GNUC__
void *memccpy(void *restrict s1, const  void  *restrict  s2, int c, size_t n);
size_t strlcpy(char *dst, const char *src, size_t dstsize);
int getopt(int argc, char * const argv[], const char *optstring);
#endif

#endif
